@@ -15,7 +15,7 @@
 // | Author: Bertrand Mansion <bmansion@mamasam.com>                     |
 // +---------------------------------------------------------------------+
 //
-// $Id: Container.php,v 1.23 2003/05/20 09:05:29 mansion Exp $
+// $Id: Container.php,v 1.30 2003/11/29 11:02:37 mansion Exp $
 
 require_once('Config.php');
 
@@ -81,17 +81,15 @@ class Config_Container {
     * @param  string  $name       Name of container object
     * @param  string  $content    Content of container object
     * @param  array   $attributes Array of attributes for container object
-    * @return object    reference to created container
     */
-    function &Config_Container($type = '', $name = '', $content = '', $attributes = null)
+    function Config_Container($type = 'section', $name = '', $content = '', $attributes = null)
     {
         $this->type       = $type;
         $this->name       = $name;
         $this->content    = $content;
         $this->attributes = $attributes;
         $this->parent     = null;
-        $this->_id        = uniqid($name.$type);
-        return $this;
+        $this->_id        = uniqid($name.$type, true);
     } // end constructor
 
     /**
@@ -104,44 +102,11 @@ class Config_Container {
     * @param  object  $target     needed if you choose 'before' or 'after' for where
     * @return object  reference to new item or Pear_Error
     */
-    function &createItem($type, $item, $content, $attributes = null, $where = 'bottom', $target = null)
+    function &createItem($type, $name, $content, $attributes = null, $where = 'bottom', $target = null)
     {
-        if ($this->type != 'section') {
-            return PEAR::raiseError('Config_Container::createItem must be called on a section type object.', null, PEAR_ERROR_RETURN);
-        }
-        if (is_null($target)) {
-            $target =& $this;
-        }
-        if (!is_object($target) || get_class($target) != 'config_container') {
-            return PEAR::raiseError('Target must be a Config_Container object in Config_Container::createItem.', null, PEAR_ERROR_RETURN);
-        }
-
-        switch ($where) {
-            case 'before':
-                $index = $target->getItemIndex();
-                break;
-            case 'after':
-                $index = $target->getItemIndex()+1;
-                break;
-            case 'top':
-                $index = 0;
-                break;
-            case 'bottom':
-                $index = -1;
-                break;
-            default:
-                return PEAR::raiseError('Use only top, bottom, before or after in Config_Container::createItem.', null, PEAR_ERROR_RETURN);
-        }
-        if (isset($index) && $index >= 0) {
-            array_splice($this->children, $index, 0, 'tmp');
-        } else {
-            $index = sizeof($this->children);
-        }
-        $itemObj =& new Config_Container($type, $item, $content, $attributes);
-        $this->children[$index] =& $itemObj;
-        $this->children[$index]->parent =& $this;
-
-        return $this->children[$index];
+        $item =& new Config_Container($type, $name, $content, $attributes);
+        $result =& $this->addItem($item, $where, $target);
+        return $result;
     } // end func &createItem
     
     /**
@@ -153,16 +118,13 @@ class Config_Container {
     */
     function &addItem(&$item, $where = 'bottom', $target = null)
     {
-        if (!is_object($item) || get_class($item) != 'config_container') {
-            return PEAR::raiseError('Config_Container::addItem must be called with a Config_Container item.', null, PEAR_ERROR_RETURN);
-        }
         if ($this->type != 'section') {
             return PEAR::raiseError('Config_Container::addItem must be called on a section type object.', null, PEAR_ERROR_RETURN);
         }
         if (is_null($target)) {
             $target =& $this;
         }
-        if (!is_object($target) || get_class($target) != 'config_container') {
+        if (get_class($target) != 'config_container') {
             return PEAR::raiseError('Target must be a Config_Container object in Config_Container::addItem.', null, PEAR_ERROR_RETURN);
         }
 
@@ -185,7 +147,7 @@ class Config_Container {
         if (isset($index) && $index >= 0) {
             array_splice($this->children, $index, 0, 'tmp');
         } else {
-            $index = sizeof($this->children);
+            $index = count($this->children);
         }
         $this->children[$index] =& $item;
         $this->children[$index]->parent =& $this;
@@ -204,8 +166,7 @@ class Config_Container {
     */
     function &createComment($content = '', $where = 'bottom', $target = null)
     {
-        $item =& $this->createItem('comment', null, $content, null, $where, $target);
-        return $item;
+        return $this->createItem('comment', null, $content, null, $where, $target);
     } // end func &createComment
 
     /**
@@ -216,8 +177,7 @@ class Config_Container {
     */
     function &createBlank($where = 'bottom', $target = null)
     {
-        $item =& $this->createItem('blank', null, null, null, $where, $target);
-        return $item;
+        return $this->createItem('blank', null, null, null, $where, $target);
     } // end func &createBlank
 
     /**
@@ -233,8 +193,7 @@ class Config_Container {
     */
     function &createDirective($name, $content, $attributes = null, $where = 'bottom', $target = null)
     {
-        $item =& $this->createItem('directive', $name, $content, $attributes, $where, $target);
-        return $item;
+        return $this->createItem('directive', $name, $content, $attributes, $where, $target);
     } // end func &createDirective
 
     /**
@@ -252,8 +211,7 @@ class Config_Container {
     */
     function &createSection($name, $attributes = null, $where = 'bottom', $target = null)
     {
-        $item =& $this->createItem('section', $name, null, $attributes, $where, $target);
-        return $item;
+        return $this->createItem('section', $name, null, $attributes, $where, $target);
     } // end func &createSection
 
     /**
@@ -267,6 +225,7 @@ class Config_Container {
     * This method can only be called on an object of type 'section'.
     * Note that root is a section.
     * This method is not recursive and tries to keep the current structure.
+    * For a deeper search, use searchPath()
     *
     * @param  string    $type        Type of item: directive, section, comment, blank...
     * @param  mixed     $name        Item name
@@ -274,13 +233,16 @@ class Config_Container {
     * @param  array     $attributes  Find item with attribute set to the given value
     * @param  int       $index       Index of the item in the returned object list. If it is not set, will try to return the last item with this name.
     * @return mixed  reference to item found or false when not found
+    * @see &searchPath()
     */
-    function &getItem($type, $name = null, $content = null, $attributes = null, $index = -1)
+    function &getItem($type = null, $name = null, $content = null, $attributes = null, $index = -1)
     {
         if ($this->type != 'section') {
             return PEAR::raiseError('Config_Container::getItem must be called on a section type object.', null, PEAR_ERROR_RETURN);
         }
-        $testFields[] = 'type';
+        if (!is_null($type)) {
+            $testFields[] = 'type';
+        }
         if (!is_null($name)) {
             $testFields[] = 'name';
         }
@@ -293,10 +255,10 @@ class Config_Container {
 
         $itemsArr = array();
         $fieldsToMatch = count($testFields);
-        for ($i = 0; $i < count($this->children); $i++) {
+        for ($i = 0, $count = count($this->children); $i < $count; $i++) {
             $match = 0;
             reset($testFields);
-            foreach($testFields as $field) {
+            foreach ($testFields as $field) {
                 if ($field != 'attributes') {
                     if ($this->children[$i]->$field == ${$field}) {
                         $match++;
@@ -327,14 +289,54 @@ class Config_Container {
                 return false;
             }
         } else {
-            if (count($itemsArr) > 0) {
-                return $itemsArr[count($itemsArr)-1];
+            if ($count = count($itemsArr)) {
+                return $itemsArr[$count-1];
             } else {
                 return false;
             }
         }
     } // end func &getItem
 
+    /**
+    * Finds a node using XPATH like format:
+    * 
+    * search format = array ( item,item,item,item)
+    * item = 'string' .. will match the <string of the xml element
+    * item = array('string',array('name'=>'xyz'))
+    *       .. will match <string name="xyz">
+    * 
+    * @param    mixed   Search path and attributes
+    * 
+    * @return   mixed   Config_Container object, array of Config_Container objects or false on failure.
+    * @access   public
+    */
+    function &searchPath($args)
+    {
+        if ($this->type != 'section') {
+            return PEAR::raiseError('Config_Container::searchPath must be called on a section type object.', null, PEAR_ERROR_RETURN);
+        }
+
+        $arg = array_shift($args);
+
+        if (is_array($arg)) {
+            $name = $arg[0];
+            $attributes = $arg[1];
+        } else {
+            $name = $arg;
+            $attributes = null;
+        }
+        // find all the matches for first..
+        $match =& $this->getItem(null, $name, null, $attributes);
+
+        if (!$match) {
+            return false;
+        }
+        if (!empty($args)) {
+            return $match->searchPath($args);
+        }
+        return $match;
+    } // end func &searchPath
+
     /**
     * Returns how many children this container has
     *
@@ -352,7 +354,7 @@ class Config_Container {
         }
         $count = 0;
         if (isset($name) && isset($type)) {
-            for ($i = 0; $i < count($this->children); $i++) {
+            for ($i = 0, $children = count($this->children); $i < $children; $i++) {
                 if ($this->children[$i]->name == $name && 
                     $this->children[$i]->type == $type) {
                     $count++;
@@ -361,7 +363,7 @@ class Config_Container {
             return $count;
         }
         if (isset($type)) {
-            for ($i = 0; $i < count($this->children); $i++) {
+            for ($i = 0, $children = count($this->children); $i < $children; $i++) {
                 if ($this->children[$i]->type == $type) {
                     $count++;
                 }
@@ -370,7 +372,7 @@ class Config_Container {
         }
         if (isset($name)) {
             // Some directives can have the same name
-            for ($i = 0; $i < count($this->children); $i++) {
+            for ($i = 0, $children = count($this->children); $i < $children; $i++) {
                 if ($this->children[$i]->name == $name) {
                     $count++;
                 }
@@ -386,7 +388,7 @@ class Config_Container {
     */
     function removeItem()
     {
-        if (is_null($this->parent)) {
+        if ($this->isRoot()) {
             return PEAR::raiseError('Cannot remove root item in Config_Container::removeItem.', null, PEAR_ERROR_RETURN);
         }
         $index = $this->getItemIndex();
@@ -406,7 +408,7 @@ class Config_Container {
         if (is_object($this->parent)) {
             // This will be optimized with Zend Engine 2
             $pchildren =& $this->parent->children;
-            for ($i = 0; $i < count($pchildren); $i++) {
+            for ($i = 0, $count = count($pchildren); $i < $count; $i++) {
                 if ($pchildren[$i]->_id == $this->_id) {
                     return $i;
                 }
@@ -424,13 +426,13 @@ class Config_Container {
     {
         if (is_object($this->parent)) {
             $pchildren =& $this->parent->children;
-            for ($i = 0; $i < count($pchildren); $i++) {
+            for ($i = 0, $count = count($pchildren); $i < $count; $i++) {
                 if ($pchildren[$i]->name == $this->name &&
                     $pchildren[$i]->type == $this->type) {
                     $obj[] =& $pchildren[$i];
                 }
             }
-            for ($i = 0; $i < count($obj); $i++) {
+            for ($i = 0, $count = count($obj); $i < $count; $i++) {
                 if ($obj[$i]->_id == $this->_id) {
                     return $i;
                 }
@@ -579,12 +581,12 @@ class Config_Container {
         if ($item === false || PEAR::isError($item)) {
             // Directive does not exist, will create one
             unset($item);
-            $item =& $this->createDirective($name, $content, null);
+            return $this->createDirective($name, $content, null);
         } else {
             // Change existing directive value
             $item->setContent($content);
+            return $item;
         }
-        return $item;
     } // end func setDirective
 
     /**
@@ -593,7 +595,7 @@ class Config_Container {
     */
     function isRoot()
     {
-        if (is_null($this->parent) && $this->name == 'root') {
+        if (is_null($this->parent)) {
             return true;
         }
         return false;
@@ -625,14 +627,15 @@ class Config_Container {
     * If the container has attributes, it will use '@' and '#'
     * index is here because multiple directives can have the same name.
     *
+    * @param    bool    $useAttr        Whether to return the attributes too
     * @return array
     */
-    function toArray()
+    function toArray($useAttr = true)
     {
         $array[$this->name] = array();
         switch ($this->type) {
             case 'directive':
-                if (count($this->attributes) > 0) {
+                if ($useAttr && count($this->attributes) > 0) {
                     $array[$this->name]['#'] = $this->content;
                     $array[$this->name]['@'] = $this->attributes;
                 } else {
@@ -640,12 +643,12 @@ class Config_Container {
                 }
                 break;
             case 'section':
-                if (count($this->attributes) > 0) {
+                if ($useAttr && count($this->attributes) > 0) {
                     $array[$this->name]['@'] = $this->attributes;
                 }
-                if (count($this->children) > 0) {
-                    for ($i = 0; $i < count($this->children); $i++) {
-                        $newArr = $this->children[$i]->toArray();
+                if ($count = count($this->children)) {
+                    for ($i = 0; $i < $count; $i++) {
+                        $newArr = $this->children[$i]->toArray($useAttr);
                         if (!is_null($newArr)) {
                             foreach ($newArr as $key => $value) {
                                 if (isset($array[$this->name][$key])) {
